"""
World of Warcraft API Wrapper
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

An asynchronous wrapper for the World of Warcraft API

:copyright: (c) 2021-Present Adalyia
:license: MIT, see LICENSE for more details.

"""

from .api import *
from .regions import *
from .wowapi import *
