"""
Classic World of Warcraft API Endpoints
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Module containing endpoint access methods for the Classic World of Warcraft API.

:copyright: (c) 2021-Present Adalyia
:license: MIT, see LICENSE for more details.

"""

from .game_data_classic import *
